# Generated by Django 5.1.1 on 2024-10-18 07:30

import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('ognajd', '0002_alter_version_id'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='version',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='versions', to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
        migrations.AddField(
            model_name='version',
            name='author_name',
            field=models.CharField(blank=True, default='', max_length=128, verbose_name='Author name'),
        ),
        migrations.AlterField(
            model_name='version',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Content type'),
        ),
        migrations.AlterField(
            model_name='version',
            name='dump',
            field=models.JSONField(editable=False, verbose_name='Dump'),
        ),
        migrations.AlterField(
            model_name='version',
            name='hash',
            field=models.CharField(max_length=32, verbose_name='Hash'),
        ),
        migrations.AlterField(
            model_name='version',
            name='id',
            field=models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False, verbose_name='UUID'),
        ),
        migrations.AlterField(
            model_name='version',
            name='index',
            field=models.IntegerField(default=0, verbose_name='Index'),
        ),
        migrations.AlterField(
            model_name='version',
            name='object_id',
            field=models.CharField(max_length=36, verbose_name='Object id'),
        ),
        migrations.AlterField(
            model_name='version',
            name='timestamp',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Timestamp'),
        ),
    ]
